@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2018 Jay Berkenbilt
+/* Copyright (c) 2005-2020 Jay Berkenbilt
  *
  * This file is part of qpdf.
  *
@@ -20,8 +20,8 @@
  * see the manual for additional information.
  */
 
-#ifndef __QPDF_C_H__
-#define __QPDF_C_H__
+#ifndef QPDF_C_H
+#define QPDF_C_H
 
 /*
  * This file defines a basic "C" API for qpdf.  It provides access to
@@ -389,6 +389,40 @@ extern "C" {
 	QPDF_BOOL allow_print, QPDF_BOOL allow_modify,
 	QPDF_BOOL allow_extract, QPDF_BOOL allow_annotate);
 
+    QPDF_DLL
+    void qpdf_set_r3_encryption_parameters2(
+	qpdf_data qpdf, char const* user_password, char const* owner_password,
+	QPDF_BOOL allow_accessibility, QPDF_BOOL allow_extract,
+        QPDF_BOOL allow_assemble, QPDF_BOOL allow_annotate_and_form,
+        QPDF_BOOL allow_form_filling, QPDF_BOOL allow_modify_other,
+	enum qpdf_r3_print_e print);
+
+    QPDF_DLL
+    void qpdf_set_r4_encryption_parameters2(
+	qpdf_data qpdf, char const* user_password, char const* owner_password,
+	QPDF_BOOL allow_accessibility, QPDF_BOOL allow_extract,
+        QPDF_BOOL allow_assemble, QPDF_BOOL allow_annotate_and_form,
+        QPDF_BOOL allow_form_filling, QPDF_BOOL allow_modify_other,
+	enum qpdf_r3_print_e print,
+        QPDF_BOOL encrypt_metadata, QPDF_BOOL use_aes);
+
+    QPDF_DLL
+    void qpdf_set_r5_encryption_parameters2(
+	qpdf_data qpdf, char const* user_password, char const* owner_password,
+	QPDF_BOOL allow_accessibility, QPDF_BOOL allow_extract,
+        QPDF_BOOL allow_assemble, QPDF_BOOL allow_annotate_and_form,
+        QPDF_BOOL allow_form_filling, QPDF_BOOL allow_modify_other,
+	enum qpdf_r3_print_e print, QPDF_BOOL encrypt_metadata);
+
+    QPDF_DLL
+    void qpdf_set_r6_encryption_parameters2(
+	qpdf_data qpdf, char const* user_password, char const* owner_password,
+	QPDF_BOOL allow_accessibility, QPDF_BOOL allow_extract,
+        QPDF_BOOL allow_assemble, QPDF_BOOL allow_annotate_and_form,
+        QPDF_BOOL allow_form_filling, QPDF_BOOL allow_modify_other,
+	enum qpdf_r3_print_e print, QPDF_BOOL encrypt_metadata);
+
+    /* Pre 8.4.0 encryption API */
     QPDF_DLL
     void qpdf_set_r3_encryption_parameters(
 	qpdf_data qpdf, char const* user_password, char const* owner_password,
@@ -442,4 +476,4 @@ extern "C" {
 #endif
 
 
-#endif /* __QPDF_C_H__ */
+#endif /* QPDF_C_H */
