@@ -1,9 +1,18 @@
 #include <qpdf/Pipeline.hh>
 #include <stdexcept>
 
+Pipeline::Members::Members(Pipeline* next) :
+    next(next)
+{
+}
+
+Pipeline::Members::~Members()
+{
+}
+
 Pipeline::Pipeline(char const* identifier, Pipeline* next) :
     identifier(identifier),
-    next(next)
+    m(new Members(next))
 {
 }
 
@@ -14,11 +23,17 @@ Pipeline::~Pipeline()
 Pipeline*
 Pipeline::getNext(bool allow_null)
 {
-    if ((next == 0) && (! allow_null))
+    if ((this->m->next == 0) && (! allow_null))
     {
 	throw std::logic_error(
 	    this->identifier +
 	    ": Pipeline::getNext() called on pipeline with no next");
     }
-    return this->next;
+    return this->m->next;
+}
+
+std::string
+Pipeline::getIdentifier() const
+{
+    return this->identifier;
 }
