@@ -56,12 +56,12 @@ QPDF::getAllPages()
 }
 
 void
-QPDF::getAllPagesInternal(QPDFObjectHandle cur_pages,
+QPDF::getAllPagesInternal(QPDFObjectHandle cur_node,
                           std::vector<QPDFObjectHandle>& result,
                           std::set<QPDFObjGen>& visited,
                           std::set<QPDFObjGen>& seen)
 {
-    QPDFObjGen this_og = cur_pages.getObjGen();
+    QPDFObjGen this_og = cur_node.getObjGen();
     if (visited.count(this_og) > 0)
     {
         throw QPDFExc(
@@ -70,23 +70,11 @@ QPDF::getAllPagesInternal(QPDFObjectHandle cur_pages,
             "Loop detected in /Pages structure (getAllPages)");
     }
     visited.insert(this_og);
-    std::string type;
-    QPDFObjectHandle type_key = cur_pages.getKey("/Type");
-    if (type_key.isName())
+    std::string wanted_type;
+    if (cur_node.hasKey("/Kids"))
     {
-        type = type_key.getName();
-    }
-    else if (cur_pages.hasKey("/Kids"))
-    {
-        type = "/Pages";
-    }
-    else
-    {
-        type = "/Page";
-    }
-    if (type == "/Pages")
-    {
-	QPDFObjectHandle kids = cur_pages.getKey("/Kids");
+        wanted_type = "/Pages";
+	QPDFObjectHandle kids = cur_node.getKey("/Kids");
 	int n = kids.getArrayNItems();
 	for (int i = 0; i < n; ++i)
 	{
@@ -108,17 +96,22 @@ QPDF::getAllPagesInternal(QPDFObjectHandle cur_pages,
 	    getAllPagesInternal(kid, result, visited, seen);
 	}
     }
-    else if (type == "/Page")
-    {
-        seen.insert(this_og);
-	result.push_back(cur_pages);
-    }
     else
     {
-	throw QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
-		      this->m->last_object_description,
-		      this->m->file->getLastOffset(),
-		      "invalid Type " + type + " in page tree");
+        wanted_type = "/Page";
+        seen.insert(this_og);
+	result.push_back(cur_node);
+    }
+
+    QPDFObjectHandle type_key = cur_node.getKey("/Type");
+    if (! (type_key.isName() && (type_key.getName() == wanted_type)))
+    {
+        warn(QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
+                     "page tree node",
+                     this->m->file->getLastOffset(),
+                     "/Type key should be " + wanted_type +
+                     " but is not; overriding"));
+        cur_node.replaceKey("/Type", QPDFObjectHandle::newName(wanted_type));
     }
     visited.erase(this_og);
 }
@@ -154,19 +147,19 @@ QPDF::flattenPagesTree()
 
     QPDFObjectHandle pages = getRoot().getKey("/Pages");
 
-    int const len = this->m->all_pages.size();
-    for (int pos = 0; pos < len; ++pos)
+    size_t const len = this->m->all_pages.size();
+    for (size_t pos = 0; pos < len; ++pos)
     {
         // populate pageobj_to_pages_pos and fix parent pointer
-        insertPageobjToPage(this->m->all_pages.at(pos), pos, true);
+        insertPageobjToPage(this->m->all_pages.at(pos), toI(pos), true);
         this->m->all_pages.at(pos).replaceKey("/Parent", pages);
     }
 
     pages.replaceKey("/Kids", QPDFObjectHandle::newArray(this->m->all_pages));
     // /Count has not changed
-    if (pages.getKey("/Count").getIntValue() != len)
+    if (pages.getKey("/Count").getUIntValue() != len)
     {
-        throw std::logic_error("/Count is wrong after flattening pages tree");
+        throw std::runtime_error("/Count is wrong after flattening pages tree");
     }
 }
 
@@ -222,26 +215,25 @@ QPDF::insertPage(QPDFObjectHandle newpage, int pos)
 
     QTC::TC("qpdf", "QPDF insert page",
             (pos == 0) ? 0 :                      // insert at beginning
-            (pos == static_cast<int>(this->m->all_pages.size())) ? 1 : // at end
+            (pos == QIntC::to_int(this->m->all_pages.size())) ? 1 : // at end
             2);                                   // insert in middle
 
     QPDFObjectHandle pages = getRoot().getKey("/Pages");
     QPDFObjectHandle kids = pages.getKey("/Kids");
-    assert ((pos >= 0) &&
-            (static_cast<size_t>(pos) <= this->m->all_pages.size()));
+    assert ((pos >= 0) && (QIntC::to_size(pos) <= this->m->all_pages.size()));
 
     newpage.replaceKey("/Parent", pages);
     kids.insertItem(pos, newpage);
     int npages = kids.getArrayNItems();
     pages.replaceKey("/Count", QPDFObjectHandle::newInteger(npages));
     this->m->all_pages.insert(this->m->all_pages.begin() + pos, newpage);
-    assert(this->m->all_pages.size() == static_cast<size_t>(npages));
+    assert(this->m->all_pages.size() == QIntC::to_size(npages));
     for (int i = pos + 1; i < npages; ++i)
     {
-        insertPageobjToPage(this->m->all_pages.at(i), i, false);
+        insertPageobjToPage(this->m->all_pages.at(toS(i)), i, false);
     }
     insertPageobjToPage(newpage, pos, true);
-    assert(this->m->pageobj_to_pages_pos.size() == static_cast<size_t>(npages));
+    assert(this->m->pageobj_to_pages_pos.size() == QIntC::to_size(npages));
 }
 
 void
@@ -250,8 +242,7 @@ QPDF::removePage(QPDFObjectHandle page)
     int pos = findPage(page); // also ensures flat /Pages
     QTC::TC("qpdf", "QPDF remove page",
             (pos == 0) ? 0 :                            // remove at beginning
-            (pos == static_cast<int>(
-                this->m->all_pages.size() - 1)) ? 1 :   // end
+            (pos == QIntC::to_int(this->m->all_pages.size() - 1)) ? 1 : // end
             2);                                         // remove in middle
 
     QPDFObjectHandle pages = getRoot().getKey("/Pages");
@@ -261,12 +252,12 @@ QPDF::removePage(QPDFObjectHandle page)
     int npages = kids.getArrayNItems();
     pages.replaceKey("/Count", QPDFObjectHandle::newInteger(npages));
     this->m->all_pages.erase(this->m->all_pages.begin() + pos);
-    assert(this->m->all_pages.size() == static_cast<size_t>(npages));
+    assert(this->m->all_pages.size() == QIntC::to_size(npages));
     this->m->pageobj_to_pages_pos.erase(page.getObjGen());
-    assert(this->m->pageobj_to_pages_pos.size() == static_cast<size_t>(npages));
+    assert(this->m->pageobj_to_pages_pos.size() == QIntC::to_size(npages));
     for (int i = pos; i < npages; ++i)
     {
-        insertPageobjToPage(this->m->all_pages.at(i), i, false);
+        insertPageobjToPage(this->m->all_pages.at(toS(i)), i, false);
     }
 }
 
@@ -291,8 +282,9 @@ QPDF::addPage(QPDFObjectHandle newpage, bool first)
     }
     else
     {
-        insertPage(newpage,
-                   getRoot().getKey("/Pages").getKey("/Count").getIntValue());
+        insertPage(
+            newpage,
+            getRoot().getKey("/Pages").getKey("/Count").getIntValueAsInt());
     }
 }
 
