@@ -1384,17 +1384,17 @@ sph_enc32be(void *dst, sph_u32 val)
 #endif
 		*(sph_u32 *)dst = val;
 	} else {
-		((unsigned char *)dst)[0] = (val >> 24);
-		((unsigned char *)dst)[1] = (val >> 16);
-		((unsigned char *)dst)[2] = (val >> 8);
-		((unsigned char *)dst)[3] = val;
+		((unsigned char *)dst)[0] = (unsigned char)(val >> 24);
+		((unsigned char *)dst)[1] = (unsigned char)(val >> 16);
+		((unsigned char *)dst)[2] = (unsigned char)(val >> 8);
+		((unsigned char *)dst)[3] = (unsigned char)(val);
 	}
 #endif
 #else
-	((unsigned char *)dst)[0] = (val >> 24);
-	((unsigned char *)dst)[1] = (val >> 16);
-	((unsigned char *)dst)[2] = (val >> 8);
-	((unsigned char *)dst)[3] = val;
+	((unsigned char *)dst)[0] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[3] = (unsigned char)(val);
 #endif
 }
 
@@ -1413,10 +1413,10 @@ sph_enc32be_aligned(void *dst, sph_u32 val)
 #elif SPH_BIG_ENDIAN
 	*(sph_u32 *)dst = val;
 #else
-	((unsigned char *)dst)[0] = (val >> 24);
-	((unsigned char *)dst)[1] = (val >> 16);
-	((unsigned char *)dst)[2] = (val >> 8);
-	((unsigned char *)dst)[3] = val;
+	((unsigned char *)dst)[0] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[3] = (unsigned char)(val);
 #endif
 }
 
@@ -1502,17 +1502,17 @@ sph_enc32le(void *dst, sph_u32 val)
 #endif
 		*(sph_u32 *)dst = val;
 	} else {
-		((unsigned char *)dst)[0] = val;
-		((unsigned char *)dst)[1] = (val >> 8);
-		((unsigned char *)dst)[2] = (val >> 16);
-		((unsigned char *)dst)[3] = (val >> 24);
+		((unsigned char *)dst)[0] = (unsigned char)(val);
+		((unsigned char *)dst)[1] = (unsigned char)(val >> 8);
+		((unsigned char *)dst)[2] = (unsigned char)(val >> 16);
+		((unsigned char *)dst)[3] = (unsigned char)(val >> 24);
 	}
 #endif
 #else
-	((unsigned char *)dst)[0] = val;
-	((unsigned char *)dst)[1] = (val >> 8);
-	((unsigned char *)dst)[2] = (val >> 16);
-	((unsigned char *)dst)[3] = (val >> 24);
+	((unsigned char *)dst)[0] = (unsigned char)(val);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 24);
 #endif
 }
 
@@ -1531,10 +1531,10 @@ sph_enc32le_aligned(void *dst, sph_u32 val)
 #elif SPH_BIG_ENDIAN
 	*(sph_u32 *)dst = sph_bswap32(val);
 #else
-	((unsigned char *)dst)[0] = val;
-	((unsigned char *)dst)[1] = (val >> 8);
-	((unsigned char *)dst)[2] = (val >> 16);
-	((unsigned char *)dst)[3] = (val >> 24);
+	((unsigned char *)dst)[0] = (unsigned char)(val);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 24);
 #endif
 }
 
@@ -1682,14 +1682,14 @@ sph_enc64be(void *dst, sph_u64 val)
 	}
 #endif
 #else
-	((unsigned char *)dst)[0] = (val >> 56);
-	((unsigned char *)dst)[1] = (val >> 48);
-	((unsigned char *)dst)[2] = (val >> 40);
-	((unsigned char *)dst)[3] = (val >> 32);
-	((unsigned char *)dst)[4] = (val >> 24);
-	((unsigned char *)dst)[5] = (val >> 16);
-	((unsigned char *)dst)[6] = (val >> 8);
-	((unsigned char *)dst)[7] = val;
+	((unsigned char *)dst)[0] = (unsigned char)(val >> 56);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 48);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 40);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 32);
+	((unsigned char *)dst)[4] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[5] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[6] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[7] = (unsigned char)(val);
 #endif
 }
 
@@ -1708,14 +1708,14 @@ sph_enc64be_aligned(void *dst, sph_u64 val)
 #elif SPH_BIG_ENDIAN
 	*(sph_u64 *)dst = val;
 #else
-	((unsigned char *)dst)[0] = (val >> 56);
-	((unsigned char *)dst)[1] = (val >> 48);
-	((unsigned char *)dst)[2] = (val >> 40);
-	((unsigned char *)dst)[3] = (val >> 32);
-	((unsigned char *)dst)[4] = (val >> 24);
-	((unsigned char *)dst)[5] = (val >> 16);
-	((unsigned char *)dst)[6] = (val >> 8);
-	((unsigned char *)dst)[7] = val;
+	((unsigned char *)dst)[0] = (unsigned char)(val >> 56);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 48);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 40);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 32);
+	((unsigned char *)dst)[4] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[5] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[6] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[7] = (unsigned char)(val);
 #endif
 }
 
@@ -1824,14 +1824,14 @@ sph_enc64le(void *dst, sph_u64 val)
 	}
 #endif
 #else
-	((unsigned char *)dst)[0] = val;
-	((unsigned char *)dst)[1] = (val >> 8);
-	((unsigned char *)dst)[2] = (val >> 16);
-	((unsigned char *)dst)[3] = (val >> 24);
-	((unsigned char *)dst)[4] = (val >> 32);
-	((unsigned char *)dst)[5] = (val >> 40);
-	((unsigned char *)dst)[6] = (val >> 48);
-	((unsigned char *)dst)[7] = (val >> 56);
+	((unsigned char *)dst)[0] = (unsigned char)(val);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[4] = (unsigned char)(val >> 32);
+	((unsigned char *)dst)[5] = (unsigned char)(val >> 40);
+	((unsigned char *)dst)[6] = (unsigned char)(val >> 48);
+	((unsigned char *)dst)[7] = (unsigned char)(val >> 56);
 #endif
 }
 
@@ -1850,14 +1850,14 @@ sph_enc64le_aligned(void *dst, sph_u64 val)
 #elif SPH_BIG_ENDIAN
 	*(sph_u64 *)dst = sph_bswap64(val);
 #else
-	((unsigned char *)dst)[0] = val;
-	((unsigned char *)dst)[1] = (val >> 8);
-	((unsigned char *)dst)[2] = (val >> 16);
-	((unsigned char *)dst)[3] = (val >> 24);
-	((unsigned char *)dst)[4] = (val >> 32);
-	((unsigned char *)dst)[5] = (val >> 40);
-	((unsigned char *)dst)[6] = (val >> 48);
-	((unsigned char *)dst)[7] = (val >> 56);
+	((unsigned char *)dst)[0] = (unsigned char)(val);
+	((unsigned char *)dst)[1] = (unsigned char)(val >> 8);
+	((unsigned char *)dst)[2] = (unsigned char)(val >> 16);
+	((unsigned char *)dst)[3] = (unsigned char)(val >> 24);
+	((unsigned char *)dst)[4] = (unsigned char)(val >> 32);
+	((unsigned char *)dst)[5] = (unsigned char)(val >> 40);
+	((unsigned char *)dst)[6] = (unsigned char)(val >> 48);
+	((unsigned char *)dst)[7] = (unsigned char)(val >> 56);
 #endif
 }
 
