@@ -2,7 +2,7 @@
 #
 # This file is part of qtest.
 #
-# Copyright 1993-2019, Jay Berkenbilt
+# Copyright 1993-2020, Jay Berkenbilt
 #
 # QTest is distributed under the terms of version 2.0 of the Artistic
 # license which may be found in the source distribution.
@@ -711,9 +711,20 @@ sub runtest
     elsif (defined $in_file)
     {
 	&QTC::TC("testdriver", "TestDriver input file");
-	open($in, '<', $in_file) or
-	    croak +(+__PACKAGE__,
-		    "->runtest: unable to read from input file $in_file: $!\n");
+	if (! open($in, '<', $in_file))
+        {
+            # If the input file is not found, generate a string to use
+            # as input, and make the string different each run to
+            # prevent sloppy commiting of the string as the expected
+            # output. This prevents complete failure in cases when the
+            # input file is not found, such as when one test's input
+            # file is generated by something that may have failed
+            # earlier.
+            my $not_found = "qtest, pid=$$: input file $in_file not found\n";
+            open($in, '<', \$not_found) or
+                croak +(+__PACKAGE__,
+                        "->runtest: unable to read from input file $in_file: $!\n");
+        }
     }
     elsif (defined $in_command)
     {
