@@ -12,8 +12,14 @@ static char const* whoami = 0;
 
 void usage()
 {
-    std::cerr << "Usage: " << whoami << " { -uncompress | -compress }"
-	      << std::endl;
+    std::cerr << "Usage: " << whoami << " { -uncompress | -compress[=n] }"
+	      << std::endl
+              << "If n is specified with -compress, it is a"
+              << " zlib compression level from" << std::endl
+              << "1 to 9 where lower numbers are faster and"
+              << " less compressed and higher" << std::endl
+              << "numbers are slower and more compressed"
+              << std::endl;
     exit(2);
 }
 
@@ -54,6 +60,12 @@ int main(int argc, char* argv[])
     {
 	action = Pl_Flate::a_deflate;
     }
+    else if ((strncmp(argv[1], "-compress=", 10) == 0))
+    {
+	action = Pl_Flate::a_deflate;
+        int level = QUtil::string_to_int(argv[1] + 10);
+        Pl_Flate::setCompressionLevel(level);
+    }
     else
     {
 	usage();
@@ -61,8 +73,9 @@ int main(int argc, char* argv[])
 
     QUtil::binary_stdout();
     QUtil::binary_stdin();
-    Pl_StdioFile* out = new Pl_StdioFile("stdout", stdout);
-    Pl_Flate* flate = new Pl_Flate("flate", out, action);
+    PointerHolder<Pl_StdioFile> out = new Pl_StdioFile("stdout", stdout);
+    PointerHolder<Pl_Flate> flate =
+        new Pl_Flate("flate", out.getPointer(), action);
 
     try
     {
@@ -70,7 +83,7 @@ int main(int argc, char* argv[])
 	bool done = false;
 	while (! done)
 	{
-	    int len = fread(buf, 1, sizeof(buf), stdin);
+	    size_t len = fread(buf, 1, sizeof(buf), stdin);
 	    if (len <= 0)
 	    {
 		done = true;
@@ -81,8 +94,6 @@ int main(int argc, char* argv[])
 	    }
 	}
 	flate->finish();
-	delete flate;
-	delete out;
     }
     catch (std::exception& e)
     {
